--[[----------------------------------------------------------------
--	CodeNameBlaze, an ElvUI edit by Blazeflack @ tukui.org
--
--	This file contains changes/additions to the ElvUI Unitframes
----------------------------------------------------------------]]--
local E, L, V, P, G, _ = unpack(ElvUI); --Import: Engine, Locales, PrivateDB, ProfileDB, GlobalDB, Localize Underscore
local UF = E:GetModule('UnitFrames')
local CNB = E:GetModule('CodeNameBlaze')
local LSM = LibStub("LibSharedMedia-3.0")
local ElvUF = ElvUI.oUF

--Add colored resting tag
ElvUF.Tags.Events['restingcolored'] = "PLAYER_UPDATE_RESTING"
ElvUF.Tags.Methods['restingcolored'] = function(unit)
	if (unit == 'player' and IsResting()) then
		return '|cff3E8AC6(|r|cff77DA73zzz|r|cFF3E8AC6)|r'
	else
		return ''
	end
end

--Set spacing between individual aura icons and inject modified PostUpdateAura function
function CNB:SetAuraSpacing()
	local units = E.db.CNB.unitframes.auras.auraSpacing.units
	local spacing = E.db.CNB.unitframes.auras.auraSpacing.spacing or 1

	for unit, unitName in pairs(UF.units) do
		local frameNameUnit = E:StringTitle(unitName)
		frameNameUnit = frameNameUnit:gsub('t(arget)', 'T%1')
		
		local unitframe = _G['ElvUF_'..frameNameUnit]
		if unitframe then
			unitframe.Buffs.spacing = units[unitName] and spacing or E.Spacing
			unitframe.Debuffs.spacing = units[unitName] and spacing or E.Spacing
			unitframe.Buffs.PostUpdateIcon = CNB.PostUpdateAura
			unitframe.Debuffs.PostUpdateIcon = CNB.PostUpdateAura
		end
		UF[unit]:Update()
	end

	for unit, unitgroup in pairs(UF.groupunits) do
		local frameNameUnit = E:StringTitle(unit)
		frameNameUnit = frameNameUnit:gsub('t(arget)', 'T%1')
		
		local unitframe = _G['ElvUF_'..frameNameUnit]
		if unitframe then
			unitframe.Buffs.spacing = units[unitgroup] and spacing or E.Spacing
			unitframe.Debuffs.spacing = units[unitgroup] and spacing or E.Spacing
			unitframe.Buffs.PostUpdateIcon = CNB.PostUpdateAura
			unitframe.Debuffs.PostUpdateIcon = CNB.PostUpdateAura
		end
		UF[unit]:Update()
	end
	
	for _, header in pairs(UF.headers) do
		local name = header.groupName
		local db = UF.db['units'][name]
		for i = 1, header:GetNumChildren() do
			local group = select(i, header:GetChildren())
			for j = 1, group:GetNumChildren() do
				local unitbutton = select(j, group:GetChildren())
				if unitbutton.Buffs then
					unitbutton.Buffs.spacing = units[name] and spacing or E.Spacing
					unitbutton.Buffs.PostUpdateIcon = CNB.PostUpdateAura
				end
				if unitbutton.Debuffs then
					unitbutton.Debuffs.spacing = units[name] and spacing or E.Spacing
					unitbutton.Debuffs.PostUpdateIcon = CNB.PostUpdateAura
				end
			end
		end
	end
	
	UF:UpdateAllHeaders()
end

local f = CreateFrame('Frame')
f:RegisterEvent('PLAYER_ENTERING_WORLD')
f:SetScript('OnEvent', function(self)
	self:UnregisterEvent('PLAYER_ENTERING_WORLD')
	CNB:SetAuraSpacing()
end)

--Reposition aura duration and count text
function CNB:UpdateAuraIconSettings(auras, noCycle)
	local frame = auras:GetParent()
	local type = auras.type
	if(noCycle) then
		frame = auras:GetParent():GetParent()
		type = auras:GetParent().type
	end
	if(not frame.db) then return end
	
	local db = frame.db[type]
	local index = 1
	if(db) then
		if(not noCycle) then
			while(auras[index]) do
				local button = auras[index]
				button.text:Point(E.db.CNB.unitframes.auras.textPosition, E.db.CNB.unitframes.auras.textOffsetX, E.db.CNB.unitframes.auras.textOffsetY)
				button.count:Point(E.db.CNB.unitframes.auras.countPosition, E.db.CNB.unitframes.auras.countOffsetX, E.db.CNB.unitframes.auras.countOffsetY)
				
				index = index + 1
			end
		else
			auras.text:Point(E.db.CNB.unitframes.auras.textPosition, E.db.CNB.unitframes.auras.textOffsetX, E.db.CNB.unitframes.auras.textOffsetY)
			auras.count:Point(E.db.CNB.unitframes.auras.countPosition, E.db.CNB.unitframes.auras.countOffsetX, E.db.CNB.unitframes.auras.countOffsetY)
		end
	end
end
hooksecurefunc(UF, "UpdateAuraIconSettings", CNB.UpdateAuraIconSettings)

--Copied from ElvUI and modified to fix overlapping borders on auras
function CNB:PostUpdateAura(unit, button, index, offset, filter, isDebuff, duration, timeLeft)
	local name, _, _, _, dtype, duration, expiration, _, isStealable = UnitAura(unit, index, button.filter)

	local isFriend = UnitIsFriend('player', unit)
	if button.isDebuff then
		if(not isFriend and button.owner ~= "player" and button.owner ~= "vehicle") --[[and (not E.isDebuffWhiteList[name])]] then
			button:SetBackdropBorderColor(0.9, 0.1, 0.1)
			button.icon:SetDesaturated((unit and not unit:find('arena%d')) and true or false)
		else
			local color = DebuffTypeColor[dtype] or DebuffTypeColor.none
			if (name == "Unstable Affliction" or name == "Vampiric Touch") and E.myclass ~= "WARLOCK" then
				button:SetBackdropBorderColor(0.05, 0.85, 0.94)
			else
				button:SetBackdropBorderColor(color.r * 0.6, color.g * 0.6, color.b * 0.6)
			end
			button.icon:SetDesaturated(false)
		end
	else
		if (isStealable) and not isFriend then
			button:SetBackdropBorderColor(237/255, 234/255, 142/255)
		else
			button:SetBackdropBorderColor(unpack(E["media"].bordercolor))
		end
	end

	local size = button:GetParent().size
	if size then
		if E.global.general.autoScale then
			button:SetSize(size, size) --:Size() uses E:Scale() which causes overlapping borders when using auto scale
		else
			button:Size(size)
		end
	end

	button.spell = name
	button.isStealable = isStealable
	button.duration = duration

	if expiration and duration ~= 0 then
		if not button:GetScript('OnUpdate') then
			button.expirationTime = expiration
			button.expiration = expiration - GetTime()
			button.nextupdate = -1
			button:SetScript('OnUpdate', UF.UpdateAuraTimer)
		end
		if (button.expirationTime ~= expiration) or (button.expiration ~= (expiration - GetTime()))  then
			button.expirationTime = expiration
			button.expiration = expiration - GetTime()
			button.nextupdate = -1
		end
	end
	if duration == 0 or expiration == 0 then
		button.expirationTime = nil
		button.expiration = nil
		button.priority = nil
		button.duration = nil
		button:SetScript('OnUpdate', nil)
		if(button.text:GetFont()) then
			button.text:SetText('')
		end
	end
end

local function SetAuraWidth(self, frame, db)
	local buffs = frame.Buffs
	local debuffs = frame.Debuffs
	if not buffs or not debuffs then return end
	buffs.SetWidth = nil
	debuffs.SetWidth = nil
	
	if db.buffs.sizeOverride and db.buffs.sizeOverride > 0 then		
		buffs:SetWidth(db.buffs.perrow * db.buffs.sizeOverride + ((db.buffs.perrow - 1) * buffs.spacing))
		buffs.SetWidth = E.noop
	end

	if db.debuffs.sizeOverride and db.debuffs.sizeOverride > 0 then
		debuffs:SetWidth(db.debuffs.perrow * db.debuffs.sizeOverride + ((db.debuffs.perrow - 1) * debuffs.spacing))
		debuffs.SetWidth = E.noop
	end
end

--Set Raid Debuffs to use main UF font and fontOutline settings
function CNB:Update_PartyFrames(frame, db)
	local rdebuffs = frame.RaidDebuffs
	if db.rdebuffs.enable and rdebuffs then
		rdebuffs.count:FontTemplate(LSM:Fetch("font", E.db['unitframe'].font), db.rdebuffs.fontSize, E.db['unitframe'].fontOutline)
		rdebuffs.time:FontTemplate(LSM:Fetch("font", E.db['unitframe'].font), db.rdebuffs.fontSize, E.db['unitframe'].fontOutline)
	end
	
	SetAuraWidth(self, frame, db)
end
hooksecurefunc(UF, 'Update_PartyFrames', CNB.Update_PartyFrames)

function CNB:Update_RaidFrames(frame, db)
	local rdebuffs = frame.RaidDebuffs
	if db.rdebuffs.enable then
		rdebuffs.count:FontTemplate(LSM:Fetch("font", E.db['unitframe'].font), db.rdebuffs.fontSize, E.db['unitframe'].fontOutline)
		rdebuffs.time:FontTemplate(LSM:Fetch("font", E.db['unitframe'].font), db.rdebuffs.fontSize, E.db['unitframe'].fontOutline)
	end
	
	SetAuraWidth(self, frame, db)
end
hooksecurefunc(UF, 'Update_RaidFrames', CNB.Update_RaidFrames)

function CNB:Update_Raid40Frames(frame, db)
	local rdebuffs = frame.RaidDebuffs
	if db.rdebuffs.enable then
		rdebuffs.count:FontTemplate(LSM:Fetch("font", E.db['unitframe'].font), db.rdebuffs.fontSize, E.db['unitframe'].fontOutline)
		rdebuffs.time:FontTemplate(LSM:Fetch("font", E.db['unitframe'].font), db.rdebuffs.fontSize, E.db['unitframe'].fontOutline)
	end
	
	SetAuraWidth(self, frame, db)
end
hooksecurefunc(UF, 'Update_Raid40Frames', CNB.Update_Raid40Frames)

function CNB:UpdateAuraTimer(elapsed)	
	local timervalue, formatid
	timervalue, formatid, self.nextupdate = E:GetTimeInfo(self.expiration, E.db.CNB.unitframes.auras.decimalThreshold)
	if self.text:GetFont() then
		self.text:SetFormattedText(("%s%s|r"):format(E.TimeColors[formatid], E.TimeFormats[formatid][2]), timervalue)
	elseif self:GetParent():GetParent().db then
		self.text:FontTemplate(LSM:Fetch("font", E.db['unitframe'].font), self:GetParent():GetParent().db[self:GetParent().type].fontSize, 'OUTLINE')
		self.text:SetFormattedText(("%s%s|r"):format(E.TimeColors[formatid], E.TimeFormats[formatid][2]), timervalue)
	end
end
hooksecurefunc(UF, "UpdateAuraTimer", CNB.UpdateAuraTimer)

hooksecurefunc(UF, 'Update_PlayerFrame', SetAuraWidth)
hooksecurefunc(UF, 'Update_TargetFrame', SetAuraWidth)
hooksecurefunc(UF, 'Update_TargetTargetFrame', SetAuraWidth)
hooksecurefunc(UF, 'Update_TargetTargetTargetFrame', SetAuraWidth)
hooksecurefunc(UF, 'Update_FocusFrame', SetAuraWidth)
hooksecurefunc(UF, 'Update_FocusTargetFrame', SetAuraWidth)
hooksecurefunc(UF, 'Update_PetFrame', SetAuraWidth)
hooksecurefunc(UF, 'Update_PetTargetFrame', SetAuraWidth)
hooksecurefunc(UF, 'Update_RaidpetFrames', SetAuraWidth)
hooksecurefunc(UF, 'Update_ArenaFrames', SetAuraWidth)
hooksecurefunc(UF, 'Update_BossFrames', SetAuraWidth)

local function UFOptions()
	E.Options.args.CNB.args.config.args.unitframes = {
		order = 3,
		type = 'group',
		name = L['Unitframes'],
		childGroups = "tab",
		args = {
			header = {
				order = 1,
				type = "header",
				name = CNB:ColorStr(L['Unitframes']),
			},
			auras = {
				order = 2,
				type = 'group',
				name = L['Auras'],
				get = function(info) return E.db.CNB.unitframes.auras[ info[#info] ] end,
				set = function(info, value) E.db.CNB.unitframes.auras[ info[#info] ] = value; E:StaticPopup_Show('CONFIG_RL'); end,
				args = {
					textPosition = {
						order = 1,
						type = 'select',
						name = L['Duration Position'],
						desc = L['Position of the duration text on the aura icon.'],
						values = {
							['BOTTOMLEFT'] = L['Bottom Left'],
							['BOTTOMRIGHT'] = L['Bottom Right'],
							['TOPLEFT'] = L['Top Left'],
							['TOPRIGHT'] = L['Top Right'],
							['CENTER'] = L['Center'],
						},
					},
					textOffsetX = {
						order = 2,
						type = 'range',
						name = L['Duration X-Offset'],
						desc = L['Horizontal offset of the duration text.'],
						min = -20, max = 20, step = 1,
					},
					textOffsetY = {
						order = 3,
						type = 'range',
						name = L['Duration Y-Offset'],
						desc = L['Vertical offset of the duration text.'],
						min = -20, max = 20, step = 1,
					},
					spacer2 = {
						order = 4,
						type = 'description',
						name = '',
					},
					countPosition = {
						order = 5,
						type = 'select',
						name = L['Count Position'],
						desc = L['Position of the stack count on the aura icon.'],
						values = {
							['BOTTOMLEFT'] = L['Bottom Left'],
							['BOTTOMRIGHT'] = L['Bottom Right'],
							['TOPLEFT'] = L['Top Left'],
							['TOPRIGHT'] = L['Top Right'],
							['CENTER'] = L['Center'],
						},
					},
					countOffsetX = {
						order = 6,
						type = 'range',
						name = L['Count X-Offset'],
						desc = L['Horizontal offset of the stack count.'],
						min = -20, max = 20, step = 1,
					},
					countOffsetY = {
						order = 7,
						type = 'range',
						name = L['Count Y-Offset'],
						desc = L['Vertical offset of the stack count.'],
						min = -20, max = 20, step = 1,
					},
					spacer = {
						order = 8,
						type = 'description',
						name = '',
					},
					decimalThreshold = {
						type = 'range',
						order = 9,
						name = L["Decimal Threshold"],
						desc = L['Threshold before the timer changes color and goes into decimal form. Set to -1 to disable.'],
						set = function(info, value) E.db.CNB.unitframes.auras[ info[#info] ] = value; end,
						min = -1, max = 30, step = 1,
					},
					spacing = {
						type = 'range',
						order = 10,
						name = L["Aura Spacing"],
						desc = L['Sets space between individual aura icons.'],
						get = function(info) return E.db.CNB.unitframes.auras.auraSpacing.spacing end,
						set = function(info, value) E.db.CNB.unitframes.auras.auraSpacing.spacing = value; CNB:SetAuraSpacing(); end,
						min = 0, max = 10, step = 1,
					},
					units = {
						type = "multiselect",
						order = 11,
						name = L["Set Aura Spacing On Following Units"],
						get = function(info, key) return E.db.CNB.unitframes.auras.auraSpacing.units[key] end,
						set = function(info, key, value) E.db.CNB.unitframes.auras.auraSpacing.units[key] = value; CNB:SetAuraSpacing(); end,
						values = {
							['player'] = L["Player"],
							['target'] = L["Target"],
							['targettarget'] = L["TargetTarget"],
							['targettargettarget'] = L["TargetTargetTarget"],
							['focus'] = L["Focus"],
							['focustarget'] = L["FocusTarget"],
							['pet'] = L["Pet"],
							['pettarget'] = L["PetTarget"],
							['arena'] = L["Arena"],
							['boss'] = L["Boss"],
							['party'] = L["Party"],
							['raid'] = L["Raid"],
							['raid40'] = L["Raid40"],
							['raidpet'] = L["RaidPet"],
						},
					},
				},
			},
		},
	}
end
CNB.configs['unitframes'] = UFOptions